import { LayoutToType, PlatformName } from "@wormhole-foundation/sdk-base";
import { ChainAddress } from "../address";
import { CircleMessageId } from "../attestation";
import "../payloads/connect";
import { RpcConnection } from "../rpc";
import { AnyAddress, TokenId } from "../types";
import { UnsignedTransaction } from "../unsignedTransaction";
export declare const circleMessageLayout: readonly [{
    readonly binary: "uint";
    readonly size: 4;
    readonly custom: 0;
    readonly omit: true;
    readonly name: "version";
}, {
    readonly binary: "uint";
    readonly size: 4;
    readonly custom: {
        readonly to: (id: number) => "Solana" | "Btc" | "Algorand" | "Sui" | "Aptos" | "Near" | "Ethereum" | "Terra" | "Bsc" | "Polygon" | "Avalanche" | "Oasis" | "Aurora" | "Fantom" | "Karura" | "Acala" | "Klaytn" | "Celo" | "Moonbeam" | "Neon" | "Terra2" | "Injective" | "Osmosis" | "Arbitrum" | "Optimism" | "Gnosis" | "Pythnet" | "Xpla" | "Base" | "Sei" | "Rootstock" | "Wormchain" | "Cosmoshub" | "Evmos" | "Kujira" | "Sepolia";
        readonly from: (name: "Ethereum" | "Avalanche" | "Arbitrum" | "Optimism" | "Base") => 0 | 2 | 1 | 3 | 6;
    };
    readonly name: "sourceDomain";
}, {
    readonly binary: "uint";
    readonly size: 4;
    readonly custom: {
        readonly to: (id: number) => "Solana" | "Btc" | "Algorand" | "Sui" | "Aptos" | "Near" | "Ethereum" | "Terra" | "Bsc" | "Polygon" | "Avalanche" | "Oasis" | "Aurora" | "Fantom" | "Karura" | "Acala" | "Klaytn" | "Celo" | "Moonbeam" | "Neon" | "Terra2" | "Injective" | "Osmosis" | "Arbitrum" | "Optimism" | "Gnosis" | "Pythnet" | "Xpla" | "Base" | "Sei" | "Rootstock" | "Wormchain" | "Cosmoshub" | "Evmos" | "Kujira" | "Sepolia";
        readonly from: (name: "Ethereum" | "Avalanche" | "Arbitrum" | "Optimism" | "Base") => 0 | 2 | 1 | 3 | 6;
    };
    readonly name: "destinationDomain";
}, {
    readonly binary: "uint";
    readonly size: 8;
    readonly name: "nonce";
}, {
    readonly binary: "bytes";
    readonly size: 32;
    readonly custom: {
        to: (val: Uint8Array) => import("..").UniversalAddress;
        from: (val: import("..").UniversalAddress) => Uint8Array;
    };
    readonly name: "sender";
}, {
    readonly binary: "bytes";
    readonly size: 32;
    readonly custom: {
        to: (val: Uint8Array) => import("..").UniversalAddress;
        from: (val: import("..").UniversalAddress) => Uint8Array;
    };
    readonly name: "recipient";
}, {
    readonly binary: "bytes";
    readonly size: 32;
    readonly custom: {
        to: (val: Uint8Array) => import("..").UniversalAddress;
        from: (val: import("..").UniversalAddress) => Uint8Array;
    };
    readonly name: "destinationCaller";
}, {
    readonly name: "payload";
    readonly binary: "object";
    readonly layout: readonly [{
        readonly binary: "uint";
        readonly size: 4;
        readonly custom: 0;
        readonly omit: true;
        readonly name: "messageBodyVersion";
    }, {
        readonly binary: "bytes";
        readonly size: 32;
        readonly custom: {
            to: (val: Uint8Array) => import("..").UniversalAddress;
            from: (val: import("..").UniversalAddress) => Uint8Array;
        };
        readonly name: "burnToken";
    }, {
        readonly binary: "bytes";
        readonly size: 32;
        readonly custom: {
            to: (val: Uint8Array) => import("..").UniversalAddress;
            from: (val: import("..").UniversalAddress) => Uint8Array;
        };
        readonly name: "mintRecipient";
    }, {
        readonly binary: "uint";
        readonly size: 32;
        readonly name: "amount";
    }, {
        readonly binary: "bytes";
        readonly size: 32;
        readonly custom: {
            to: (val: Uint8Array) => import("..").UniversalAddress;
            from: (val: import("..").UniversalAddress) => Uint8Array;
        };
        readonly name: "messageSender";
    }];
}];
export declare const deserializeCircleMessage: (data: Uint8Array) => [LayoutToType<typeof circleMessageLayout>, string];
export type CircleTransferMessage = {
    from: ChainAddress;
    to: ChainAddress;
    token: TokenId;
    amount: bigint;
    messageId: CircleMessageId;
};
export interface AutomaticCircleBridge<P extends PlatformName> {
    transfer(sender: AnyAddress, recipient: ChainAddress, amount: bigint, nativeGas?: bigint): AsyncGenerator<UnsignedTransaction>;
}
export interface CircleBridge<P extends PlatformName> {
    redeem(sender: AnyAddress, message: string, attestation: string): AsyncGenerator<UnsignedTransaction>;
    transfer(sender: AnyAddress, recipient: ChainAddress, amount: bigint): AsyncGenerator<UnsignedTransaction>;
    parseTransactionDetails(txid: string): Promise<CircleTransferMessage>;
}
export interface SupportsCircleBridge<P extends PlatformName> {
    getCircleBridge(rpc: RpcConnection<P>): Promise<CircleBridge<P>>;
}
export declare function supportsCircleBridge<P extends PlatformName>(thing: SupportsCircleBridge<P> | any): thing is SupportsCircleBridge<P>;
export interface SupportsAutomaticCircleBridge<P extends PlatformName> {
    getAutomaticCircleBridge(rpc: RpcConnection<P>): Promise<AutomaticCircleBridge<P>>;
}
export declare function supportsAutomaticCircleBridge<P extends PlatformName>(thing: SupportsAutomaticCircleBridge<P> | any): thing is SupportsAutomaticCircleBridge<P>;
//# sourceMappingURL=cctp.d.ts.map