"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTransactionIdentifier = exports.nativeChainAddress = exports.isTokenId = void 0;
const sdk_base_1 = require("@wormhole-foundation/sdk-base");
const address_1 = require("./address");
const signer_1 = require("./signer");
const universalAddress_1 = require("./universalAddress");
function isTokenId(thing) {
    return (typeof thing.address !== undefined &&
        (0, sdk_base_1.isChain)(thing.chain));
}
exports.isTokenId = isTokenId;
function nativeChainAddress(s) {
    if (Array.isArray(s)) {
        // We might be passed a universal address as a string
        // First try to decode it as native, otherwise try
        // to decode it as universal and convert it to native
        let address;
        try {
            address = (0, address_1.toNative)(s[0], s[1]);
        }
        catch {
            address =
                s[1] instanceof universalAddress_1.UniversalAddress
                    ? s[1].toNative(s[0])
                    : new universalAddress_1.UniversalAddress(s[1]).toNative(s[0]);
        }
        return {
            chain: s[0],
            address: address,
        };
    }
    if ((0, signer_1.isSigner)(s))
        return {
            chain: s.chain(),
            address: (0, address_1.toNative)(s.chain(), s.address()),
        };
    return {
        chain: s.chain,
        address: s.address.toNative(s.chain),
    };
}
exports.nativeChainAddress = nativeChainAddress;
function isTransactionIdentifier(thing) {
    return (thing.chain !== undefined &&
        thing.txid !== undefined);
}
exports.isTransactionIdentifier = isTransactionIdentifier;
//# sourceMappingURL=types.js.map