import { Layout, LayoutToType, DynamicItemsOfLayout } from "@wormhole-foundation/sdk-base";
declare global {
    namespace Wormhole {
        interface PayloadLiteralToLayoutMapping {
        }
    }
}
export type LayoutLiteral = keyof Wormhole.PayloadLiteralToLayoutMapping & string;
type LayoutOf<LL extends LayoutLiteral> = LL extends infer V extends LayoutLiteral ? Wormhole.PayloadLiteralToLayoutMapping[V] : never;
type LayoutLiteralToPayloadType<LL extends LayoutLiteral> = LayoutToType<LayoutOf<LL>>;
type ProtocolName = string | null;
type ToLiteralFormat<PN extends ProtocolName, PayloadName extends string> = PN extends null ? PayloadName : `${PN}:${PayloadName}`;
type ComposeLiteral<ProtocolN extends ProtocolName, PayloadN extends string, Literal> = ToLiteralFormat<ProtocolN, PayloadN> extends infer L extends Literal ? L : never;
export declare const composeLiteral: <ProtocolN extends ProtocolName, PayloadN extends string>(protocol: ProtocolN, payloadName: PayloadN) => ComposeLiteral<ProtocolN, PayloadN, PayloadLiteral>;
export type PayloadLiteral = LayoutLiteral | "Uint8Array";
type PayloadLiteralToPayloadType<PL extends PayloadLiteral> = PL extends LayoutLiteral ? LayoutLiteralToPayloadType<PL> : Uint8Array;
type DecomposeLiteral<PL extends PayloadLiteral> = PL extends `${infer Protocol}:${infer LayoutName}` ? [Protocol, LayoutName] : [null, PL];
declare const baseLayout: readonly [{
    readonly name: "version";
    readonly binary: "uint";
    readonly size: 1;
    readonly custom: 1;
    readonly omit: true;
}, {
    readonly binary: "uint";
    readonly size: 4;
    readonly name: "guardianSet";
}, {
    readonly name: "signatures";
    readonly binary: "array";
    readonly lengthSize: 1;
    readonly arrayItem: {
        readonly binary: "object";
        readonly layout: readonly [{
            readonly name: "guardianIndex";
            readonly binary: "uint";
            readonly size: 1;
        }, {
            readonly binary: "bytes";
            readonly size: 65;
            readonly custom: {
                readonly to: (val: Uint8Array) => import("./signature").Signature;
                readonly from: (val: import("./signature").Signature) => Uint8Array;
            };
            readonly name: "signature";
        }];
    };
}, {
    readonly name: "timestamp";
    readonly binary: "uint";
    readonly size: 4;
}, {
    readonly name: "nonce";
    readonly binary: "uint";
    readonly size: 4;
}, {
    readonly custom: {
        to: (val: number) => "Solana" | "Btc" | "Algorand" | "Sui" | "Aptos" | "Near" | "Ethereum" | "Terra" | "Bsc" | "Polygon" | "Avalanche" | "Oasis" | "Aurora" | "Fantom" | "Karura" | "Acala" | "Klaytn" | "Celo" | "Moonbeam" | "Neon" | "Terra2" | "Injective" | "Osmosis" | "Arbitrum" | "Optimism" | "Gnosis" | "Pythnet" | "Xpla" | "Base" | "Sei" | "Rootstock" | "Wormchain" | "Cosmoshub" | "Evmos" | "Kujira" | "Sepolia";
        from: (val: "Solana" | "Btc" | "Algorand" | "Sui" | "Aptos" | "Near" | "Ethereum" | "Terra" | "Bsc" | "Polygon" | "Avalanche" | "Oasis" | "Aurora" | "Fantom" | "Karura" | "Acala" | "Klaytn" | "Celo" | "Moonbeam" | "Neon" | "Terra2" | "Injective" | "Osmosis" | "Arbitrum" | "Optimism" | "Gnosis" | "Pythnet" | "Xpla" | "Base" | "Sei" | "Rootstock" | "Wormchain" | "Cosmoshub" | "Evmos" | "Kujira" | "Sepolia") => number;
    };
    readonly binary: "uint";
    readonly size: 2;
    readonly name: "emitterChain";
}, {
    readonly binary: "bytes";
    readonly size: 32;
    readonly custom: {
        to: (val: Uint8Array) => import("./universalAddress").UniversalAddress;
        from: (val: import("./universalAddress").UniversalAddress) => Uint8Array;
    };
    readonly name: "emitterAddress";
}, {
    readonly binary: "uint";
    readonly size: 8;
    readonly name: "sequence";
}, {
    readonly name: "consistencyLevel";
    readonly binary: "uint";
    readonly size: 1;
}];
type BaseLayout = LayoutToType<typeof baseLayout>;
export interface VAA<PL extends PayloadLiteral = PayloadLiteral> extends BaseLayout {
    readonly protocolName: DecomposeLiteral<PL>[0];
    readonly payloadName: DecomposeLiteral<PL>[1];
    readonly payloadLiteral: PL;
    readonly payload: PayloadLiteralToPayloadType<PL>;
    readonly hash: Uint8Array;
}
export type DistributiveVAA<PL extends PayloadLiteral> = PL extends PayloadLiteral ? VAA<PL> : never;
export type ProtocolVAA<PN extends ProtocolName, PayloadName extends string> = ComposeLiteral<PN, PayloadName, PayloadLiteral> extends infer PL extends PayloadLiteral ? DistributiveVAA<PL> : never;
type PayloadLiteralToDynamicItems<PL extends PayloadLiteral> = DynamicItemsOfLayout<[
    ...typeof baseLayout,
    PL extends LayoutLiteral ? {
        name: "payload";
        binary: "object";
        layout: DynamicItemsOfLayout<LayoutOf<PL>>;
    } : {
        name: "payload";
        binary: "bytes";
    }
]>;
export declare function createVAA<PL extends PayloadLiteral = "Uint8Array">(payloadLiteral: PL, vaaData: LayoutToType<PayloadLiteralToDynamicItems<PL>>): VAA<PL>;
export declare function registerPayloadType(protocol: ProtocolName, name: string, layout: Layout): void;
type AtLeast1<T> = readonly [T, ...T[]];
type AtLeast2<T> = readonly [T, T, ...T[]];
export type NamedPayloads = AtLeast1<readonly [string, Layout]>;
export type RegisterPayloadTypes<ProtocolN extends ProtocolName, NP extends NamedPayloads> = {
    readonly [E in NP[number] as ToLiteralFormat<ProtocolN, E[0]>]: E[1];
};
export declare function registerPayloadTypes(protocol: ProtocolName, payloads: NamedPayloads): void;
export declare function serialize<PL extends PayloadLiteral>(vaa: VAA<PL>): Uint8Array;
export declare function serializePayload<PL extends PayloadLiteral>(payloadLiteral: PL, payload: PayloadLiteralToPayloadType<PL>): Uint8Array;
export type Byteish = Uint8Array | string;
export type PayloadDiscriminator<LL extends LayoutLiteral = LayoutLiteral, AllowAmbiguous extends boolean = false> = (data: Byteish) => AllowAmbiguous extends false ? LL | null : readonly LL[];
type PayloadGroup = readonly [ProtocolName, AtLeast1<string>];
type PayloadGroupToLayoutLiterals<PG extends PayloadGroup> = ComposeLiteral<PG[0], PG[1][number], LayoutLiteral>;
type PayloadGroupsToLayoutLiteralsRecursive<PGA extends readonly PayloadGroup[]> = PGA extends readonly [
    infer PG extends PayloadGroup,
    ...infer T extends readonly PayloadGroup[]
] ? PayloadGroupToLayoutLiterals<PG> | PayloadGroupsToLayoutLiteralsRecursive<T> : never;
type PayloadGroupsToLayoutLiterals<PGA extends readonly PayloadGroup[]> = PayloadGroupsToLayoutLiteralsRecursive<PGA> extends infer Value extends LayoutLiteral ? Value : never;
type LLDtoLLs<LLD extends AtLeast2<LayoutLiteral> | readonly [ProtocolName, AtLeast2<string>] | AtLeast2<PayloadGroup>> = LLD extends AtLeast2<LayoutLiteral> ? LLD[number] : LLD extends readonly [ProtocolName, AtLeast2<string>] ? PayloadGroupToLayoutLiterals<LLD> : LLD extends AtLeast2<PayloadGroup> ? PayloadGroupsToLayoutLiterals<LLD> : never;
export declare function payloadDiscriminator<const LLD extends AtLeast2<LayoutLiteral> | readonly [ProtocolName, AtLeast2<string>] | AtLeast2<PayloadGroup>, B extends boolean = false>(payloadLiterals: LLD, allowAmbiguous?: B): PayloadDiscriminator<LLDtoLLs<LLD>, B>;
export type ExtractLiteral<T> = T extends PayloadDiscriminator<infer LL> ? LL : T;
export declare function deserialize<T extends PayloadLiteral | PayloadDiscriminator>(payloadDet: T, data: Byteish): DistributiveVAA<ExtractLiteral<T>>;
type DeserializedPair<LL extends LayoutLiteral = LayoutLiteral> = LL extends LayoutLiteral ? readonly [LL, LayoutLiteralToPayloadType<LL>] : never;
type DeserializePayloadReturn<T> = T extends PayloadLiteral ? PayloadLiteralToPayloadType<T> : T extends PayloadDiscriminator<infer LL> ? DeserializedPair<LL> : never;
export declare function deserializePayload<T extends PayloadLiteral | PayloadDiscriminator>(payloadDet: T, data: Byteish, offset?: number): DeserializePayloadReturn<T>;
export declare const blindDeserializePayload: (data: Byteish) => readonly DeserializedPair[];
export {};
//# sourceMappingURL=vaa.d.ts.map