import { ChainName, PlatformName } from "@wormhole-foundation/sdk-base";
import { NativeAddress } from "./address";
import { WormholeMessageId } from "./attestation";
import { Platform } from "./platform";
import { AutomaticCircleBridge, CircleBridge } from "./protocols/cctp";
import { IbcBridge } from "./protocols/ibc";
import { AutomaticTokenBridge, TokenBridge } from "./protocols/tokenBridge";
import { RpcConnection } from "./rpc";
import { ChainConfig, SignedTx } from "./types";
import { UniversalAddress } from "./universalAddress";
export declare abstract class ChainContext<P extends PlatformName> {
    readonly config: ChainConfig;
    abstract platform: Platform<P>;
    readonly chain: ChainName;
    protected rpc?: RpcConnection<P>;
    protected tokenBridge?: TokenBridge<P>;
    protected autoTokenBridge?: AutomaticTokenBridge<P>;
    protected circleBridge?: CircleBridge<P>;
    protected autoCircleBridge?: AutomaticCircleBridge<P>;
    protected ibcBridge?: IbcBridge<P>;
    constructor(config: ChainConfig);
    getRpc(): Promise<RpcConnection<P>>;
    getDecimals(token: NativeAddress<P> | UniversalAddress | "native"): Promise<bigint>;
    getBalance(walletAddr: string, token: NativeAddress<P> | UniversalAddress | "native"): Promise<bigint | null>;
    getCurrentBlock(): Promise<number>;
    parseTransaction(txid: string): Promise<WormholeMessageId[]>;
    sendWait(stxns: SignedTx): Promise<string[]>;
    supportsTokenBridge: () => boolean;
    getTokenBridge(): Promise<TokenBridge<P>>;
    supportsAutomaticTokenBridge: () => boolean;
    getAutomaticTokenBridge(): Promise<AutomaticTokenBridge<P>>;
    supportsCircleBridge: () => boolean;
    getCircleBridge(): Promise<CircleBridge<P>>;
    supportsAutomaticCircleBridge: () => boolean;
    getAutomaticCircleBridge(): Promise<AutomaticCircleBridge<P>>;
    supportsIbcBridge: () => boolean;
    getIbcBridge(): Promise<IbcBridge<P>>;
}
//# sourceMappingURL=chain.d.ts.map