import { PlatformName } from "@wormhole-foundation/sdk-base";
import { NativeAddress, ChainAddress } from "../address";
import { AnyAddress, TokenId } from "../types";
import { ProtocolVAA } from "../vaa";
import { UnsignedTransaction } from "../unsignedTransaction";
import "../payloads/tokenBridge";
import { RpcConnection } from "../rpc";
export declare const ErrNotWrapped: (token: string) => Error;
export interface SupportsTokenBridge<P extends PlatformName> {
    getTokenBridge(rpc: RpcConnection<P>): Promise<TokenBridge<P>>;
}
export declare function supportsTokenBridge<P extends PlatformName>(thing: SupportsTokenBridge<P> | any): thing is SupportsTokenBridge<P>;
export interface SupportsAutomaticTokenBridge<P extends PlatformName> {
    getAutomaticTokenBridge(rpc: RpcConnection<P>): Promise<AutomaticTokenBridge<P>>;
}
export declare function supportsAutomaticTokenBridge<P extends PlatformName>(thing: SupportsAutomaticTokenBridge<P> | any): thing is SupportsAutomaticTokenBridge<P>;
export declare namespace TokenBridge {
    type VAA<PayloadName extends string> = ProtocolVAA<"TokenBridge", PayloadName>;
    const getTransferDiscriminator: () => import("../vaa").PayloadDiscriminator<"TokenBridge:Transfer" | "TokenBridge:TransferWithPayload", false>;
}
export interface TokenBridge<P extends PlatformName> {
    isWrappedAsset(nativeAddress: AnyAddress): Promise<boolean>;
    getOriginalAsset(nativeAddress: AnyAddress): Promise<TokenId>;
    getWrappedNative(): Promise<NativeAddress<P>>;
    hasWrappedAsset(foreignToken: TokenId): Promise<boolean>;
    getWrappedAsset(foreignToken: TokenId): Promise<NativeAddress<P>>;
    isTransferCompleted(vaa: TokenBridge.VAA<"Transfer" | "TransferWithPayload">): Promise<boolean>;
    createAttestation(token_to_attest: AnyAddress, payer?: AnyAddress): AsyncGenerator<UnsignedTransaction>;
    submitAttestation(vaa: TokenBridge.VAA<"AttestMeta">, payer?: AnyAddress): AsyncGenerator<UnsignedTransaction>;
    transfer(sender: AnyAddress, recipient: ChainAddress, token: AnyAddress, amount: bigint, payload?: Uint8Array): AsyncGenerator<UnsignedTransaction>;
    redeem(sender: AnyAddress, vaa: TokenBridge.VAA<"Transfer" | "TransferWithPayload">, unwrapNative?: boolean): AsyncGenerator<UnsignedTransaction>;
}
export interface AutomaticTokenBridge<P extends PlatformName> {
    transfer(sender: AnyAddress, recipient: ChainAddress, token: AnyAddress, amount: bigint, relayerFee: bigint, nativeGas?: bigint): AsyncGenerator<UnsignedTransaction>;
    redeem(sender: AnyAddress, vaa: TokenBridge.VAA<"TransferWithPayload">): AsyncGenerator<UnsignedTransaction>;
    getRelayerFee(sender: ChainAddress, recipient: ChainAddress, token: TokenId | "native"): Promise<bigint>;
}
//# sourceMappingURL=tokenBridge.d.ts.map