import { encoding } from "@wormhole-foundation/sdk-base";
import { toNative } from "./address";
export class UniversalAddress {
    static byteSize = 32;
    type = "Universal";
    address;
    constructor(address) {
        if (typeof address === "string") {
            if (!UniversalAddress.isValidAddress(address))
                throw new Error(`Invalid Wormhole address, expected ${UniversalAddress.byteSize}-byte ` +
                    `hex string but got ${address}`);
            this.address = encoding.hex.decode(address);
        }
        else {
            this.address = address;
        }
    }
    toNative(platform) {
        return toNative(platform, this);
    }
    unwrap() {
        return this.address;
    }
    toString() {
        return encoding.hex.encode(this.address, true);
    }
    toUint8Array() {
        return this.address;
    }
    toUniversalAddress() {
        return this;
    }
    equals(other) {
        if (UniversalAddress.instanceof(other)) {
            return other.toString() === this.toString();
        }
        return false;
    }
    static isValidAddress(address) {
        return (encoding.hex.valid(address) &&
            encoding.stripPrefix("0x", address).length ===
                UniversalAddress.byteSize * 2);
    }
    static instanceof(address) {
        return address.type === "Universal";
    }
}
//# sourceMappingURL=universalAddress.js.map